@@ -15,7 +15,8 @@ from strategy_api.tm_api.api.rest.rest_client import RestClient, Request, Respon
 from strategy_api.tm_api.api.websocket.websocket_client import WebsocketClient
 from strategy_api.tm_api.base import BaseGateway
 from strategy_api.tm_api.object import TickData, BarData, Interval, OrderRequest, OrderData, Direction, OrderType, \
-    Offset, Status, CancelRequest, HistoryRequest, PositionSide, DataType, Product, Exchange
+    Offset, Status, CancelRequest, HistoryRequest, PositionSide, DataType, Product, Exchange, DepthData, BidData, \
+    AskData, TdMode
 
 # http 基础地址
 from strategy_api.tm_api.tools import get_order_type, symbol_deal
@@ -90,8 +91,10 @@ class BinanceFutureUsdtGateway(BaseGateway):
     exchange: Exchange = Exchange.BINANCE
     product: Product = Product.U_FUTURES
     symbolMap: dict = dict()
+    SYMBOL_SET: set = set()
 
-    def __init__(self, event_engine: EventEngine, order_switch: bool = False, bar_switch: bool = False, tick_switch: bool = False) -> None:
+    def __init__(self, event_engine: EventEngine, order_switch: bool = False, bar_switch: bool = False,
+                 tick_switch: bool = False) -> None:
         """init"""
         super().__init__(event_engine, order_switch, bar_switch, tick_switch)
         self.trade_ws_api: "BinanceUsdtTradeWebsocketApi" = BinanceUsdtTradeWebsocketApi(self)
@@ -146,6 +149,8 @@ class BinanceFutureUsdtGateway(BaseGateway):
             stop_profit: bool = False,  # 止盈
             stop_profit_price: float = 0,  # 止盈价
             position_side: PositionSide = PositionSide.ONEWAY,
+            tdMode: TdMode = TdMode.CROSS,
+            ccy: str = ""
             ):
         new_symbol = symbol_deal(symbol, self.exchange)
         self.symbolMap[new_symbol] = symbol
@@ -158,7 +163,9 @@ class BinanceFutureUsdtGateway(BaseGateway):
             price=price,
             volume=volume,
             positionSide=position_side,
-            exchange=self.exchange
+            exchange=self.exchange,
+            stop_loss_price=stop_loss_price,
+            stop_profit_price=stop_profit_price,
         )
         self.send_order(original_req, orderid)
 
@@ -174,6 +181,8 @@ class BinanceFutureUsdtGateway(BaseGateway):
              stop_profit: bool = False,  # 止盈
              stop_profit_price: float = 0,  # 止盈价
              position_side: PositionSide = PositionSide.ONEWAY,
+             tdMode: TdMode = TdMode.CROSS,
+             ccy: str = ""
              ):
         new_symbol = symbol_deal(symbol, self.exchange)
         self.symbolMap[new_symbol] = symbol
@@ -186,7 +195,9 @@ class BinanceFutureUsdtGateway(BaseGateway):
             price=price,
             volume=volume,
             positionSide=position_side,
-            exchange=self.exchange
+            exchange=self.exchange,
+            stop_loss_price=stop_loss_price,
+            stop_profit_price=stop_profit_price,
         )
         self.send_order(original_req, orderid)
 
@@ -202,6 +213,8 @@ class BinanceFutureUsdtGateway(BaseGateway):
               stop_profit: bool = False,  # 止盈
               stop_profit_price: float = 0,  # 止盈价
               position_side: PositionSide = PositionSide.ONEWAY,
+              tdMode: TdMode = TdMode.CROSS,
+              ccy: str = ""
               ):
         new_symbol = symbol_deal(symbol, self.exchange)
         self.symbolMap[new_symbol] = symbol
@@ -214,7 +227,9 @@ class BinanceFutureUsdtGateway(BaseGateway):
             price=price,
             volume=volume,
             positionSide=position_side,
-            exchange=self.exchange
+            exchange=self.exchange,
+            stop_loss_price=stop_loss_price,
+            stop_profit_price=stop_profit_price,
         )
         self.send_order(original_req, orderid)
 
@@ -230,6 +245,8 @@ class BinanceFutureUsdtGateway(BaseGateway):
               stop_profit: bool = False,  # 止盈
               stop_profit_price: float = 0,  # 止盈价
               position_side: PositionSide = PositionSide.ONEWAY,
+              tdMode: TdMode = TdMode.CROSS,
+              ccy: str = ""
               ):
         new_symbol = symbol_deal(symbol, self.exchange)
         self.symbolMap[new_symbol] = symbol
@@ -242,7 +259,9 @@ class BinanceFutureUsdtGateway(BaseGateway):
             price=price,
             volume=volume,
             positionSide=position_side,
-            exchange=self.exchange
+            exchange=self.exchange,
+            stop_loss_price=stop_loss_price,
+            stop_profit_price=stop_profit_price,
         )
         self.send_order(original_req, orderid)
 
@@ -263,11 +282,21 @@ class BinanceFutureUsdtGateway(BaseGateway):
         self.rest_api.cancel_order(req)
 
     # 获取历史K线
-    def query_history(self, symbol: str, interval: Interval, hour: int = 0, minutes: int = 0) -> List[BarData]:
+    def query_history(self,
+                      symbol: str,
+                      interval: Interval,
+                      hour: int = 0,
+                      minutes: int = 0,
+                      end_time: datetime = None
+                      ) -> List[BarData]:
         new_symbol = symbol_deal(symbol, self.exchange)
         self.symbolMap[new_symbol] = symbol
 
-        end: datetime = datetime.now()
+        if end_time:
+            end: datetime = end_time
+        else:
+            end: datetime = datetime.now()
+
         if hour:
             start: datetime = end - timedelta(hours=hour)
         elif minutes:
@@ -285,6 +314,24 @@ class BinanceFutureUsdtGateway(BaseGateway):
         )
         return self.rest_api.query_history(req)
 
+    def query_depth(self, symbol: str, limit: int) -> DepthData:
+        new_symbol = symbol_deal(symbol, self.exchange)
+        self.symbolMap[new_symbol] = symbol
+
+        depth_data: DepthData = DepthData(
+            symbol=symbol,
+            bid_data=[],
+            ask_data=[]
+        )
+
+        if limit not in [5, 10, 20, 50, 100, 500, 1000, 5000]:
+            print(f"无该档位: {limit}")
+            return depth_data
+        return self.rest_api.query_depth(new_symbol, limit, depth_data)
+
+    def query_symbol(self, quote_symbol: str, s_type: str = "") -> list:
+        return [i.replace(quote_symbol.upper(), "-" + quote_symbol.upper()) for i in self.SYMBOL_SET if i.endswith(quote_symbol.upper())]
+
     # 订阅数据
     def subscribe(self, symbol: str, data_type: DataType, interval: Interval = None) -> None:
         new_symbol = symbol_deal(symbol, self.exchange)
@@ -300,7 +347,6 @@ class BinanceFutureUsdtGateway(BaseGateway):
         else:
             print(f"订阅失败，未知数据类型: {data_type}")
 
-
     # 关闭
     def close(self) -> None:
         self.rest_api.stop()
@@ -395,11 +441,27 @@ class BinanceUsdtRestApi(RestClient):
         self.start()
 
         # 获取服务器时间
+        self.query_contract()
         self.query_time()
         # 保持用户 ws 的链接
         if self.gateway.order_call_switch:
             self.start_user_stream()
 
+    def query_contract(self) -> None:
+        data: dict = {
+            "security": Security.NONE
+        }
+        self.add_request(
+            method="GET",
+            path="/fapi/v1/exchangeInfo",
+            callback=self.on_query_contract,
+            data=data
+        )
+
+    def on_query_contract(self, data: dict, request: Request) -> None:
+        for d in data["symbols"]:
+            self.gateway.SYMBOL_SET.add(d['symbol'])
+
     # 查询服务器时间
     def query_time(self) -> None:
         data: dict = {
@@ -434,7 +496,7 @@ class BinanceUsdtRestApi(RestClient):
         order: OrderData = req.create_order_data(
             orderid,
         )
-        self.gateway.on_order(order)
+        self.gateway.on_order(copy(order))
 
         data: dict = {
             "security": Security.SIGNED
@@ -591,6 +653,40 @@ class BinanceUsdtRestApi(RestClient):
 
         return history
 
+    def query_depth(self, symbol: str, limit: int, depth_data: DepthData) -> DepthData:
+        params: dict = {
+            "symbol": symbol,
+            "limit": limit
+        }
+
+        resp: Response = self.request(
+            "GET",
+            "/api/v3/depth",
+            data={"security": Security.NONE},
+            params=params
+        )
+        if resp.status_code // 100 != 2:
+            msg: str = f"查询depth数据失败, status code：{resp.status_code}，msg：{resp.text}"
+            print(msg)
+        else:
+            data: dict = resp.json()
+
+            for bid in data["bids"]:
+                bid_data: BidData = BidData(
+                    bid_price=bid[0],
+                    bid_volume=bid[1]
+                )
+                depth_data.bid_data.append(bid_data)
+
+            for ask in data["asks"]:
+                ask_data: AskData = AskData(
+                    ask_price=ask[0],
+                    ask_volume=ask[1]
+                )
+                depth_data.ask_data.append(ask_data)
+
+        return depth_data
+
     # 开始ws 交易数据推送
     def start_user_stream(self) -> None:
         """获取ws listen key 帮助重新连接ws """
@@ -705,7 +801,7 @@ class BinanceUsdtRestApi(RestClient):
             order.rejected_reason = request.response.text if request.response.text else ""
             self.gateway.on_order(order)
 
-            msg: str = f"send order failed, orderid: {order.orderid}, status code：{status_code}, msg：{request.response.text}"
+            msg: str = f"发送订单失败, orderid: {order.orderid}, status code：{status_code}, msg：{request.response.text}"
             print(msg)
 
     # 发送订单错误回调
@@ -825,19 +921,21 @@ class BinanceUsdtTradeWebsocketApi(WebsocketClient):
         ord_data: dict = packet["o"]
         key: Tuple[str, str] = (ord_data["o"], ord_data["f"])
         order_type: OrderType = ORDERTYPE_BINANCES2VT.get(key, OrderType.LIMIT)
-        price = float(ord_data["p"])
+        price = float(ord_data["p"])  # 订单原始价格
         if price <= 0:
-            price = float(ord_data['ap'])
+            price = float(ord_data['ap'])  # 订单平均价格
 
         order: OrderData = OrderData(
-            symbol=ord_data["s"],
-            orderid=str(ord_data["c"]),
+            symbol=ord_data["s"],  # 交易对
+            exchange=Exchange.BINANCE,
+            orderid=str(ord_data["c"]),  # 客户端自定订单ID
             type=order_type,
-            direction=DIRECTION_BINANCES2VT[ord_data["S"]],
+            direction=DIRECTION_BINANCES2VT[ord_data["S"]],  # 订单方向
             price=price,
-            volume=float(ord_data["q"]),
-            traded=float(ord_data["z"]),
-            traded_price=float(ord_data.get("L", "0")),
+            volume=float(ord_data["q"]),  # 订单原始数量
+            traded=float(ord_data["z"]),  # 订单累计已成交量
+            traded_volume=float(ord_data.get("l", "0")),  # 订单末次成交量
+            traded_price=float(ord_data.get("L", "0")),  # 订单末次成交价格
             status=STATUS_BINANCES2VT.get(ord_data["X"], Status.NOTTRADED),
             datetime=generate_datetime(packet["E"]),
         )
@@ -925,7 +1023,6 @@ class BinanceUsdtDataWebsocketApi(WebsocketClient):
             )
             self.ticks[target] = tick
 
-
         req: dict = {
             "method": "SUBSCRIBE",
             "params": [target],
